###################################################################################################
# _importable.py
#
# Implementation of class Importable (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from App.Common import package_home
from Globals import HTMLFile
import os
import string
import tempfile
import urllib
from cStringIO import StringIO
# Product Imports.
import _fileutil
import _globals


# -------------------------------------------------------------------------------------------------
#  importContent
# -------------------------------------------------------------------------------------------------
def importContent(self, xml, xmlfilename, REQUEST={}):
  message = ''
  
  # Process root.
  self.dTagStack = _globals.MyStack()
  self.dValueStack = _globals.MyStack()
  self.oParent = self.getParentNode()
  
  # parse XML-file.
  root = self.parse(StringIO(xml),self,1)
  
  # remove helpers after import
  self.recurse_afterImport()
  
  # Upload resource files (files and images) from zip-archive.
  zipfolder = _fileutil.getFilePath(xmlfilename)
  message = message + self.recurse_uploadRessources(zipfolder)
  
  # Commit changes.
  self.onChangeObj(REQUEST)
  
  # Return with message.
  return message


###################################################################################################
###################################################################################################
###
###   class Importable
###
###################################################################################################
###################################################################################################
class Importable:

    # Management Permissions.
    # -----------------------
    __ac_permissions__=(
		('Import/Export objects', ('manage_importexport','manage_import','manage_export',) ),
		)

    # Interface.
    # ----------
    manage_importexport = HTMLFile('dtml/zmscontainerobject/manage_importexport', globals()) 


    ###############################################################################################    
    #  Importable.manage_import:
    #
    #	
    ###############################################################################################    
    def manage_import(self, file, lang, manage_lang, REQUEST, RESPONSE=None):
      """ Importable.manage_import """
      
      message = ''
      
      # Create temporary folder.
      folder = tempfile.mktemp()
      os.mkdir(folder)
      
      # Save to temporary file.
      zipfilename = _fileutil.getOSPath('%s/%s'%(folder,_fileutil.extractFileName(file.filename)))
      _fileutil.exportObj(file,zipfilename)

      # Find xml file.      
      xmlfilename = None
      if _fileutil.extractFileExt(file.filename) == 'zip':
        _fileutil.extractZipArchive(zipfilename)
        for ext in ['xml','htm','html']:
          if xmlfilename is None:
            xmlfilename = _fileutil.findExtension(ext,folder,deep=0)
        for ext in ['xml','htm','html']:
          if xmlfilename is None:
            xmlfilename = _fileutil.findExtension(ext,folder,deep=1)
      else:
        xmlfilename = zipfilename

      if xmlfilename is None:
        message = 'XML-File not found!'
        
      else:
      
        # Apply XSL stylesheet of foreign DTD.
        dtd = REQUEST.get('dtd','') 
        if len(dtd) > 0:
          outfilename = xmlfilename + "~"
          xslt = self.getXslTransformation(dtd)
          
          # HTML TIDY
          if _fileutil.extractFileExt(xmlfilename) in ['htm','html']:
            path = self.getConfProperty('InstalledProducts.tidy','')
            conf = _fileutil.getOSPath(package_home(globals())+'/conf/xsl/tidy.html2zms.conf')
            cmd = '%s -asxml -config %s -m %s'%(path,conf,xmlfilename)
            os.system(cmd)
            
          # Write XSLT-transformation to file.
          dtdfilename = _fileutil.getOSPath('%s/%s'%(folder,xslt.id()))
          _fileutil.exportObj(xslt,dtdfilename)
          
          # Process XSLT-transformation.
          cmd = self.getConfProperty('InstalledProducts.xslt','')
          cmd = string.replace(cmd,'{trans.xsl}',dtdfilename)
          cmd = string.replace(cmd,'{in.xml}',xmlfilename)
          cmd = string.replace(cmd,'{out.xml}',outfilename)
          os.system(cmd)
          
          xmlfilename = outfilename
        
        # Read content of XML-file.
        f = open(xmlfilename,'r')
        xml = f.read()
        f.close()
        
        # Import XML.
        message = message + importContent(self,xml,xmlfilename,REQUEST)
        
        message = message + self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%_fileutil.extractFileName(file.filename))
      
      # Remove temporary files.
      _fileutil.remove(folder,deep=1)
      
      # Return with message.
      if RESPONSE:
        message = urllib.quote(message)
        return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    # ---------------------------------------------------------------------------------------------
    #  Importable.recurse_afterImport:
    #
    #  Remove helpers after XML-Import.
    # ---------------------------------------------------------------------------------------------
    def recurse_afterImport(self):
      self._recurse_afterImport()
      # Process children.
      for ob in self.getChildNodes():
        ob.recurse_afterImport()

###################################################################################################
