###################################################################################################
# _multilangmanager.py
#
# Implementation of class _multilangmanager.MultiLanguageManager (see below).
# Mehrsprachigkeit der Inhalte
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from App.version_txt import version_txt
from Globals import HTML, HTMLFile
import string
import urllib
# Product Imports.
import _xmllib


# -------------------------------------------------------------------------------------------------
#  importLangDict
# -------------------------------------------------------------------------------------------------

def _importLangDict(self, item):
  key = item['key']
  lang_dict = self.get_lang_dict()
  lang_dict[key] = {}
  for s_lang in self.getLanguages():
    lang_dict[key][s_lang] = item[s_lang]
  self.setConfProperty('ZMS.custom.langs.dict',lang_dict.copy())

def importLangDict(self, xml):
  builder = _xmllib.XmlAttrBuilder()
  value = builder.parse(xml)
  if type(value) is type([]):
    for item in value:
      _importLangDict(self,item)
  else:
    _importLangDict(self,value)


###################################################################################################
###################################################################################################
###
###   C l a s s   M u l t i L a n g u a g e M a n a g e r
###
###################################################################################################
###################################################################################################
class MultiLanguageManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
                'manage_customizeLanguagesForm','manage_changeLanguagesProperties',
                )
    __ac_permissions__=(
                ('ZMS Administrator', __administratorPermissions__),
                ('ZMS Author', __authorPermissions__),
                ('ZMS Editor', __editorPermissions__),
                )


    # Management Interface to customize Languages.
    manage_customizeLanguagesForm = HTMLFile('dtml/multilangmanager/manage_customizelanguagesform', globals()) 


    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.recurse_updateVersionLanguages: 
    #
    # Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersionLanguages(self, REQUEST):
      old_langs = self.__get_languages__()
      new_langs = {}
      for id in old_langs.keys():
        if type(old_langs[id]) is type({}):
          new_langs[id] = old_langs[id].copy()
        elif type(old_langs[id]) is type(''):
          new_langs[id] = {}
          new_langs[id]['label'] = old_langs[id]
          if id == self.getPrimaryLanguage():
            new_langs[id]['parent'] = ''
          else:
            new_langs[id]['parent'] = self.getPrimaryLanguage()
      self.attr_languages = {}
      for id in new_langs.keys():
        label = new_langs[id]['label']
        parent = new_langs[id]['parent']
        self.__set_lang__(id,label,parent)


    """
    ###############################################################################################
    #	P R I M A R Y   L A N G U A G E
    ###############################################################################################
    """

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.setPrimaryLanguage: 
    #
    # Sets ID of the primary language.
    # ------------------------------------------------------------------------------------------
    def setPrimaryLanguage(self, lang):
      self.language_primary = lang

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.getPrimaryLanguage: 
    #
    # Returns ID of the primary language.
    # ------------------------------------------------------------------------------------------
    def getPrimaryLanguage(self):
      return self.language_primary

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.getParentLanguage: 
    #
    # Returns ID of parent language.
    # ------------------------------------------------------------------------------------------
    def getParentLanguage(self, id):
      parent = None
      langs = self.__get_languages__()
      if langs.has_key(id):
        lang = langs[id]
        parent = lang['parent']
      return parent

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.getDescendantLanguages: 
    #
    # Returns IDs of descendant languages.
    # ------------------------------------------------------------------------------------------
    def getDescendantLanguages(self, id):
      descendants = []
      langs = self.__get_languages__()
      for descendant in langs.keys():
        lang = langs[descendant]
        if lang['parent'] == id:
          descendants.append(descendant)
      return descendants

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.getManageLanguage: 
    #
    # Returns ID of the preferred language for the management interface.
    # ------------------------------------------------------------------------------------------
    def getManageLanguage(self, id):
      languages = self.__get_languages__()[id]
      if type(languages) is type({}) and languages.has_key('manage'):
        rtn = languages['manage']
      elif id in self.getManageLanguages():
        rtn = id
      elif id == 'de':
        rtn = 'ger'
      else:
        rtn = 'eng'
      return rtn

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.__get_languages__:
    # 
    # Returns dictionary of languages.
    # ------------------------------------------------------------------------------------------
    def __get_languages__(self):
      if not hasattr(self,'attr_languages'):
        self.attr_languages = {}
      return getattr(self,'attr_languages')

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.getLanguageLabel: 
    #
    # Returns language-label of specified ID.
    # ------------------------------------------------------------------------------------------
    def getLanguageLabel(self, id):
      label = id
      langs = self.__get_languages__()
      if langs.has_key(id):
        label = langs[id]['label']
      return label

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.getLanguages: 
    #
    # Returns list of IDs of languages (primary language 1st).
    # ------------------------------------------------------------------------------------------
    def getLanguages(self, REQUEST=None):
        #
        user_langs = ['*']
        if REQUEST is not None and REQUEST.has_key('AUTHENTICATED_USER'):
          user = REQUEST['AUTHENTICATED_USER']
          user_langs = getattr(user,'langs',user_langs)
        if type(user_langs) != type([]):
          user_langs = [user_langs]
        #
        sort_langs = []
        for id in self.__get_languages__().keys():
          label = self.getLanguageLabel(id)
          if (len(user_langs)==1 and user_langs[0]=='*') or id in user_langs:
            label = self.getLanguageLabel(id)
            if id == self.getPrimaryLanguage():
              label = '*'
            sort_langs.append((label,id))
        sort_langs.sort()
        #
        langs = []
        for sort_lang in sort_langs:
          langs.append(sort_lang[1])
        #
        return langs

    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.getLanguage: 
    #
    # Get requested language of specified URL (used by index_html).
    # ------------------------------------------------------------------------------------------
    def getLanguage(self, REQUEST): 
      lang = REQUEST.get('lang',self.getPrimaryLanguage())
      url = REQUEST['URL']
      path = url
      i = string.rfind(url,'.')
      if i > 0:
        path = url[:i]
      j = string.rfind(path,'_')
      if j > 0:
        suffix = path[j+1:]
        if suffix in self.getLanguages():
          lang = suffix
      return lang

    # ------------------------------------------------------------------------------------------
    #	MultiLanguageManager._set_method: 
    # 
    #	Set/add DTML-Method for specified language.
    # ------------------------------------------------------------------------------------------
    def _set_method(self, id, raw):
        # remove existing method            
        try: self.manage_delObjects(ids=[id])
        except: pass
        # add new method
        title = '*** DO NOT DELETE OR MODIFY ***'
        self.manage_addDTMLMethod(id,title,raw)

    # ------------------------------------------------------------------------------------------
    #	MultiLanguageManager.__set_lang__: 
    # 
    #	Set/add language with specified values.
    # ------------------------------------------------------------------------------------------
    def __set_lang__(self, id, label, parent, mnge=None):
    
    	if len(parent) == 0:
    	  primLangOld = self.getPrimaryLanguage()
    	  for lang_id in self.__get_languages__().keys():
    	    if lang_id != id and self.getParentLanguage(lang_id) == '':
              attr_languages = self.__get_languages__()
              attr_languages[lang_id]['parent'] = id
              self.attr_languages = attr_languages.copy()
    	  self.setPrimaryLanguage(id)
        
        # Set language.
        # -------------
        attr_languages = self.__get_languages__()
        attr_languages[id] = {}
        attr_languages[id]['label'] = label
        attr_languages[id]['parent'] = parent
        if mnge is not None: attr_languages[id]['manage'] = mnge
        self.attr_languages = attr_languages.copy()
        
        # Set/add Standard DTML-Methods.
        # ------------------------------
        self._set_method('index_%s.html'%id,'<dtml-var index_html>')
        self._set_method('sitemap_%s.html'%id,'<dtml-call "REQUEST.set(\'op\',\'sitemap\')"><dtml-var index_html>')
        self._set_method('index_print_%s.html'%id,'<dtml-call "REQUEST.set(\'op\',\'print\')"><dtml-var index_html>')
        self._set_method('search_%s.html'%id,'<dtml-call "REQUEST.set(\'op\',\'search\')"><dtml-var index_html>')
        
        # catalog.
        # --------
        oCatalog = self.getCatalog()
        if oCatalog is not None:
	  key = 'zcat_text_%s'%id
	  if not key in oCatalog.schema(): oCatalog.manage_addColumn(key)
	  if not key in oCatalog.indexes(): oCatalog.manage_addIndex(key,'TextIndex')


    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.__del_lang__: 
    # 
    # Delete language with specified ID.
    # ------------------------------------------------------------------------------------------
    def __del_lang__(self, id):
    
        # Delete language.
        # ----------------
        dctLanguages = self.__get_languages__()
        self.attr_languages = {}
        for lang_id in dctLanguages.keys():
          if lang_id != id:
            self.attr_languages[lang_id] = dctLanguages[lang_id]
        # Update attribute.
        self.attr_languages = self.attr_languages.copy()
        
        # Remove Standard DTML-Methods.
        # -----------------------------
        sRight = '_%s.html'%id
        oids = []
        for oid in self.objectIds(['DTML Method']):
            if oid[-len(sRight):] == sRight: oids.append(oid)
        try: self.manage_delObjects(oids)
        except: pass
            
        # catalog.
        # --------
        oCatalog = self.getCatalog()
        if oCatalog is not None:
	  key = 'zcat_text_%s'%id
	  if key in oCatalog.schema(): 
            if version_txt() >= 'Zope 2.5':
              oCatalog.manage_delColumn([key])
            else:
              oCatalog.manage_delColumns([key])
	  if key in oCatalog.indexes(): 
              if version_txt() >= 'Zope 2.5':
                oCatalog.manage_delIndex([key])
              else:
                oCatalog.manage_deleteIndex([key])


    ###############################################################################################
    # MultiLanguageManager.manage_changeLanguagesProperties:
    #
    # Einstellungen fr Sprachen ndern.
    ###############################################################################################
    def manage_changeLanguagesProperties(self, lang, manage_lang, REQUEST, RESPONSE):
        """ MultiLanguageManager.manage_changeLanguagesProperties """
        
        # Delete.
        # -------
        if REQUEST['btn'] == self.getLangStr('BTN_DELETE',manage_lang):
          if REQUEST.has_key('ids'):
            for id in REQUEST['ids']:
              self.__del_lang__(id) 
        
        # Change.
        # -------
        elif REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
          for id in self.getLanguages():
            lbl = REQUEST['%s_label'%id]
            prnt = REQUEST['%s_parent'%id]
            mnge = REQUEST['%s_manage'%id]
            self.__set_lang__(id,lbl,prnt,mnge)
        
        # Insert.
        # -------
        elif REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
          id = REQUEST['language_id']
          lbl = REQUEST['language_label']
          if len(self.getLanguages()) == 0:
            prnt = ''
          else:
            prnt = REQUEST['language_parent']
          mnge = REQUEST['language_manage']
          self.__set_lang__(id,lbl,prnt,mnge)
        
        # Return with message.
        message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
        return RESPONSE.redirect('manage_customizeLanguagesForm?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.get_lang_dict: 
    #
    # Returns language-dictionary.
    # ------------------------------------------------------------------------------------------
    def get_lang_dict(self):
      return self.getConfProperty('ZMS.custom.langs.dict',{})
      
    # ------------------------------------------------------------------------------------------
    # MultiLanguageManager.getLangDict: 
    #
    # Returns list of entries from language-dictionary (ordered by key).
    # ------------------------------------------------------------------------------------------
    def getLangDict(self):
      lang_dict = self.get_lang_dict()
      lang_list = []
      keys = lang_dict.keys()
      keys.sort()
      for key in keys:
        dict = lang_dict[key]
        dict['key'] = key
        lang_list.append(dict)
      return lang_list
    
    ###############################################################################################
    # MultiLanguageManager.manage_changeLangDictProperties:
    #
    # Einstellungen fr multilinguales Wrterbuch ndern.
    ###############################################################################################
    def manage_changeLangDictProperties(self, lang, manage_lang, REQUEST, RESPONSE):
        """ MultiLanguageManager.manage_changeLangDictProperties """

        # Delete.
        # -------
        if REQUEST['btn'] == self.getLangStr('BTN_DELETE',manage_lang):
          ids = REQUEST.get('ids',[])
          dict = self.get_lang_dict()
          lang_dict = {}
          for id in dict.keys():
            if not id in ids:
              lang_dict[id] = dict[id]
          self.setConfProperty('ZMS.custom.langs.dict',lang_dict.copy())
        
        # Change.
        # -------
        elif REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
          dict = self.get_lang_dict()
          lang_dict = {}
          for id in dict.keys():
            key = string.strip(REQUEST['%s_key'%(id)])    
            lang_dict[key] = {}
            for s_lang in self.getLanguages():
              lang_dict[key][s_lang] = string.strip(REQUEST['%s_value_%s'%(id,s_lang)])
          self.setConfProperty('ZMS.custom.langs.dict',lang_dict.copy())
        
        # Insert.
        # -------
        elif REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
          id = ''
          key = string.strip(REQUEST['%s_key'%(id)])
          lang_dict = self.get_lang_dict()
          lang_dict[key] = {}
          for s_lang in self.getLanguages():
            lang_dict[key][s_lang] = string.strip(REQUEST['%s_value_%s'%(id,s_lang)])
          self.setConfProperty('ZMS.custom.langs.dict',lang_dict.copy())

        # Export.
        # -------
        elif REQUEST['btn'] == self.getLangStr('BTN_EXPORT',manage_lang):
          value = []
          ids = REQUEST.get('ids',[])
          dict = self.get_lang_dict()
          for id in dict.keys():
            item = dict[id]
            if id in ids or len(ids) == 0:
              value.append(item)
          if len(value)==1:
            value = value[0]
          content_type = 'text/xml'
          filename = 'ZMS.custom.langs.dict.xml'
          export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
          export = export + _xmllib.toXml(self,value,1)
          RESPONSE.setHeader('Content-Type',content_type)
          RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
          return export

        # Import.
        # -------
        elif REQUEST['btn'] == self.getLangStr('BTN_IMPORT',manage_lang):
          importLangDict(self,xml=REQUEST['file'])
          message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%REQUEST['file'].filename)

        # Return with message.
        message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
        return RESPONSE.redirect('manage_customizeLanguagesForm?lang=%s&manage_lang=%s&manage_tabs_message=%s#langdict'%(lang,manage_lang,message))
        
###################################################################################################
